---
title: "Mid-sleep Standard Deviation"
output: html_document
---

##### Definition

The standard deviation of the mid-sleep time gives a measure of the variation in sleep pattern.

⚠ Mid-sleep standard deviation is only valid if there is a single sleep episode per night. Check there are no sessions shown in the compliance tab.

##### Interpretation

Lower values indicate a more regular sleep pattern.
